c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: bem.F,v 1.5 2001/02/06 19:09:19 nakano Exp $
c
      subroutine bem (ndim, n, r, z, ibc, bv, bdv,
     $                b, g, h, ipd, bferr, bberr, dw, iw, ho, hp)
c
c     Solve Poisson's equation by Boudary Element Method
c     in cyrindrical-symmetry chamber using constant elements
c
c     === input ===
c     ndim   ...dimension of matrices
c     n      ...number of boundary elements
c     r(n+1) ...R coordinate of the i-th boundary node
c     z(n+1) ...Z coordinate of the i-th boundary node
c     
c       i-th boudary means the region between
c       i-th node and (i+1)-th node.
c
c     ibc(n)...=0 if Dirichlet boudary condition
c              =1 if natural boudary condition
c     bv(n) ...boudary value (for node ibc(i) = 0)
c              or derivative (for node ibc(i) = 1)
c     b(n)  ...additional vecotor originated from sources in 
c              internal space
c     
c     === output ===
c     bv(n)  ... solution at the i-th node on the boudary
c     bdv(n) ... derivative at the i-th node on the boundary
c     
c     === work array ===
c     g(ndim, n), h(ndim,n)   matrices for linear equations
c     ipd(n)                  index vecters for pivoting
c     --- for LAPACK routines ---
c     dw(3,n) ... real*8
c     iw(n)   ... integer
c     ho(ndim,n), hp(ndim.n): real*8
c
      integer n, ndim
      real*8 r(n+1), z(n+1)
      integer ibc(n)
      real*8 bv(n), bdv(n), b(n), bferr(n), bberr(n)
      real*8 g(ndim,n), h(ndim,n)
      integer ipd(n), iw(n), info
      real*8 dw(3,n)
      real*8 ho(ndim,n), hp(ndim,n)

      integer i, j

#ifdef DEBUG
#include "hisput.h"
      integer ndimmax
      parameter (ndimmax = NODEMAX)
      real*8 go(ndimmax, ndimmax)
      real*8 lhs, rhs
#endif

      call bmform(ndim, n, r, z, g, h)

#ifdef DEBUG
      do 900 j=1,n
         do 900 i = 1, n
            go(i,j) = g(i,j)
 900  continue

      write(0,*) ' G:'
      do 910 i = 1, n
         write(0,1000) (g(i,j),j=1,n)
 910  continue

      write(0,*) ' H:'
      do 920 i = 1, n
         write(0,1000) (h(i,j),j=1,n)
 920  continue
 1000 format(1x, 45F13.6)
#endif /* DEBUG */

      call pivttq(ndim, n, g, h, ibc, bv, b)

      do 10 j = 1, n
         do 10 i = 1, n
            ho(i,j) = h(i,j)
 10   continue
            
      call dgetrf(n, n, h, ndim, ipd, info)

#ifdef DEBUG
      write(0,2000) info
 2000 format(1x, '(bem) -- after dgetrf, info = ', i3)
#endif

      do 20 j = 1, n
         do 20 i = 1, n
            hp(i,j) = h(i,j)
 20   continue

      do 25 i = 1, n
         bdv(i) = b(i)
 25   continue
      call dgetrs('N', n, 1, h, ndim, ipd, bdv, ndim, info)
#ifdef DEBUG
      write(0,2001) info
 2001 format(1x, '(bem) -- after dgetrs, info = ', i3)
#endif

      call dgerfs('N', n, 1, ho, ndim, hp, ndim, ipd,
     $     b, n, bdv, n, bferr, bberr, dw, iw, info)
#ifdef DEBUG
      write(0,2001) info
 2002 format(1x, '(bem) -- after dgerfs, info = ', i3)
#endif

      call boundr(n, ibc, bv, bdv)

#ifdef DEBUG
      do 940 i = 1, n
         lhs = 0
         rhs = 0
         do 930 j = 1, n
            lhs = lhs + ho(i,j) * bv(j)
            rhs = rhs + go(i,j) * bdv(j)
 930     continue
         write(0,1100) i, lhs, rhs, lhs-rhs
 1100    format(' row: ', I3, 3F13.6)
 940  continue
#endif

      return
      end

c===============================================

      subroutine bmform(ndim, n, r, z, g, h)
c
c     create matrix to be solved
c
      integer ndim, n 
      real*8 r(n+1), z(n+1)
      real*8 g(ndim,n), h(ndim,n)
      real*8 g0, h0

      integer i, j
c
c     prepare table for double-exponential integration
c

#ifdef DEFINT_ELMOFF
      call fiab
#endif

      do 10 i = 1, n
         call elmdia(n, r, z, i, g0)
         g(i,i) = g0
         h(i,i) = 0.0d0

         do 20 j = 1, n
            if (i .ne. j) then
               call elmoff(n, r, z, i, j, g0, h0)
               g(i,j) = g0
               h(i,j) = h0
               h(i,i) = h(i,i) - h(i,j)
            end if
 20      continue
 10   continue
      
      return
      end
c===============================================

      subroutine elmoff(n, r, z, i, j, g0, h0)
c
c     compute (i,j)-th off-diagonal element of matrices a and b in form
c
      integer n, i, j
      real*8 r(n+1), z(n+1)
      real*8 g0, h0

      real*8 rb0, rb1, rs, zb0, zb1, zs, lj
      real*8 tsi, qsi
      external tsi, qsi

      real*8 xi0, xi1, v, eps
      parameter (xi0 = -1.0d0, xi1 = 1.0d0)
#ifdef DEFINT_ELMOFF
      integer l
      parameter (l = 0, eps = 1.0d-7)
#else
      real*8 epsi, esterr
      parameter (epsi = 1.0d-7)

      eps = epsi
#endif

      rb0 = dble(r(j))
      rb1 = dble(r(j+1))
      zb0 = dble(z(j))
      zb1 = dble(z(j+1))

      rs = 0.5d0 * dble(r(i) + r(i+1))
      zs = 0.5d0 * dble(z(i) + z(i+1))

      call preptq (rs, zs, rb0, zb0, rb1, zb1, lj)

#ifdef DEFINT_ELMOFF
      call defint(tsi, xi0, xi1, eps, l, v)
#else
      call nc8(tsi, xi0, xi1, eps, v, esterr)
#endif
      g0 = 0.5d0 * lj * v

#ifdef DEFINT_ELMOFF
      call defint(qsi, xi0, xi1, eps, l, v)
#else
      call nc8(qsi, xi0, xi1, eps, v, esterr)
#endif
      h0 = 0.5d0 * lj * v

      return
      end
c===============================================

      subroutine elmdia(n, r, z, i, g0)
c
c     compute (i,i)-th diagonal element of matrices a and b in form
c
c     input:
c       n      ... number of boundary elements 
c       r(n+1) ... radial position of nodes
c       z(n+1) ... axial position of nodes
c       i      ... index of elements
c
c     output:
c       g0     ... value G(i,i) for constant boundary element
c
c     work:
c       lj     ... length of i-th element
c       li     ... length to be analytical approx. can be applied
c
      real*8 arint
      parameter (arint = 25.0d0)
      integer n, i
      real*8 r(n+1), z(n+1)
      real*8 g0

      real*8 rs, rb0, rb1, ri0, ri1, dr
      real*8 zs, zb0, zb1, zi0, zi1, dz
      real*8 lj, li, lrat, lbi, sina

      real*8 ca, cb, cc, cd, g1, g2

      real*8 tsi
      external tsi
      real*8 xi0, xi1, v, eps
      parameter (xi0 = -1.0d0, xi1 = 1.0d0)
#ifdef DEFINT_ELMDIA
      integer l
      parameter (l = 0, eps = 1.0d-7)
#else
      real*8 epsi, esterr
      parameter (epsi = 1.0d-7)

      eps = epsi
#endif

      rb0 = dble(r(i))
      rb1 = dble(r(i+1))
      rs  = 0.5d0 * (rb0 + rb1)
      dr = rb1 - rb0

      zb0 = dble(z(i))
      zb1 = dble(z(i+1))
      zs  = 0.5d0 * (zb0 + zb1)
      dz  = zb1 - zb0

      lj = sqrt(dr**2 + dz**2)

      sina = - dr / lj

      li = 1.0d0 / arint * rs
     $     * (sqrt(4 * arint + sina**2) - abs(sina) )

      g1 = 0.0d0
      if (li .lt. lj) then
         lrat = li / lj

         ri0 = (1-lrat) * rs + lrat * rb0
         zi0 = (1-lrat) * zs + lrat * zb0

         ri1 = (1-lrat) * rs + lrat * rb1
         zi1 = (1-lrat) * zs + lrat * zb1

         call preptq (rs, zs, ri0, zi0, rb0, zb0, lbi)
#ifdef DEFINT_ELMDIA
         call defint(tsi, xi0, xi1, eps, l, v)
#else
         call nc8(tsi, xi0, xi1, eps, v, esterr)
#endif
         g1 = g1 + 0.5d0 * lbi * v

         call preptq (rs, zs, ri1, zi1, rb1, zb1, lbi)
#ifdef DEFINT_ELMDIA
         call defint(tsi, xi0, xi1, eps, l, v)
#else
         call nc8(tsi, xi0, xi1, eps, v, esterr)
#endif
         g1 = g1 + 0.5d0 * lbi * v
      else
         li = lj
         ri0 = rb0
         ri1 = rb1
      end if

#ifdef DEBUG
      write(0,*) ' (elmdia) node ', i
      write(0,*) ' b0 : ', rb0, zb0
      write(0,*) ' i0 : ', ri0, zi0
      write(0,*) ' i1 : ', ri1, zi1
      write(0,*) ' b1 : ', rb1, zb1
#endif

      ca = rs

      if (rb0 .eq. rb1) then
         g2 = 2 * li * (1 - log (li / (16 * ca) ) )
      else
         cb =  - 0.5d0 * (ri1 - ri0)
         cc = ca + cb
         cd = ca - cb

         g2 = log( abs( (sqrt(cc)+sqrt(ca))/(sqrt(cc)-sqrt(ca)) ) )
     $      - log( abs( (sqrt(cd)+sqrt(ca))/(sqrt(cd)-sqrt(ca)) ) )

         g2 = ( 2.0d0 / 3 - log(li**2/(256*ca)) )
     $        * (cc**1.5d0 - cd**1.5d0)
     $        + cc**1.5d0 * log(cc) - cd**1.5d0 * log(cd)
     $        + 4 * ca * (sqrt(cc) - sqrt(cd))
     $        - 2 * ca**1.5d0 * g2
         g2 = g2 * li / (3 * cb * sqrt(ca))

      end if

      g0 = g1 + g2
      return
      end
c===============================================

      subroutine pivttq(ndim, n, g, h, ibc, bv, b)
c
c     pivot matrices g, h according to ibc
c
c     original is  Hij tj = Gij qj - Bi
c
c
c
c     input
c       ndim      ... dimension of matrices and vectors
c       n         ... number of nodes
c       g(ndim,n) ... G matrix for q^*
c       h(ndim,n) ... H matrix for theta^*
c       ibc(n)    ...=0 if Dirichlet boudary condition
c                    =1 if natural boudary condition
c       bv(n)     ...boudary value (for node ibc(i) = 0)
c                    or derivative (for node ibc(i) = 1)
c       b(n)      ...additional vecotor originated from sources in 
c                    internal space
c     
c     output
c       g(ndim,n) ...matrix of linear eq.
c       b(n)      ...vector for the constant part of linear eq.
c
      integer ndim, n
      real*8 g(ndim,n), h(ndim,n)
      integer ibc(n)
      real*8 bv(n), b(n)

      integer i, j
      real*8 rswap, t

      do 10 j = 1, n
         if (ibc(j) .eq. 0) then
            do 510 i = 1, n
               rswap =  - h(i,j)
               h(i,j) = - g(i,j)
               g(i,j) = rswap
 510        continue
         end if
 10   continue

      do 520 i = 1, n
         t = 0.0d0
         do 530 j = 1, n
#ifdef DEBUG
            write(0,*) ' (pivttq): i,j', i, j
            write(0,*) ' h(i,j), bv(j):', g(i,j), bv(j)
#endif

            t = t + g(i,j) * bv(j)
 530     continue
         b(i) = - b(i) + t
 520  continue

      return
      end
c===============================================

      subroutine boundr(n, ibc, bv, bdv)
c
c     pivot back to original information
c
c     input:
c       n     ... number of nodes
c       ibc(n)...=0 if Dirichlet boudary condition
c                =1 if natural boudary condition
c       bv(n) ...boudary value (for node ibc(i) = 0)
c                or derivative (for node ibc(i) = 1)
c       bdv(n)...solved after the BEM calculation
c                derivative (for node ibc(i) = 0)
c                value      (for node ibc(i) = 1)
c
      integer n
      integer ibc(n)
      real*8 bv(n), bdv(n)

      integer j
      real*8 rswap

      do 510 j = 1, n
         if (ibc(j) .eq. 1) then
            rswap = bv(j)
            bv(j) = bdv(j)
            bdv(j) = rswap
         endif
 510  continue

      return
      end

