c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: coldist.F,v 1.3 2001/01/13 08:00:00 nakano Exp $
c
c     routines to obtain the colliding gas speed
c
c     for theoretical backgrounds, see:
c
c     T. Nakano, I. Mori and S. Baba;
c     Appl. Surf. Sci., 113/114 (1997) 642.
c
#include "hisput.h"


c=============
      subroutine setdist(pgas, tgas, mgas, rmax, vpar, mfp)
c
c     initialization subroutine for functions distf
c
c     inputs
c       pgas  ... gas Pressure (Pa)
c       tgas  ... gas temperature (K)
c       mgas  ... weight of gas molecule (kg)
c       rmax  ... maximum collision parameter (m)
c       vpar  ... particle speed (m/s)
c
c     output
c       mfp   ... mean free path of the particle
c
c     variables
c       alpha ... m/(2kT) of gas
c       rerf  ... \frac{\sqrt{(\pi)}}{2}
c       salph ... sqrt(m/2kT) for gas
c       cpar  ... particle velocity in sqrt(alpha) unit
c     (in the following, C=cpar)
c       prm_a ... \frac{2}{3}C^3 + 2C
c       prm_b ... 2C
c       prm_c ... C\exp{(-C^2)} + (2C^2+1)\frac{\sqrt{\pi}}{2}\erf(C)
c       prm_d ... (2C^2+1)\frac{\sqrt{\pi}}{2}\erf(C)
c       prm_e ... (2C^2+1)
c       prm_rc... 1/prm_c
c
      real*8 pgas, tgas, mgas, rmax, vpar
      real*8 mfp

      real*8 alpha, rerf

      real*8 unitbz, pi
      parameter (unitbz = UNITBZ)
      parameter (pi     = PIDBL)

      real*8 derf

      real*8 salpha, cpar
      real*8 prm_a, prm_b, prm_c, prm_d, prm_e, prm_rc
      common /ccoldf/ salpha, cpar,
     $     prm_a, prm_b, prm_c, prm_d, prm_e, prm_rc 


      rerf = dsqrt(pi) / 2
      
      alpha = mgas / ( 2 * unitbz * tgas )
      salpha = sqrt(alpha)
      cpar = vpar * salpha

      prm_e = 2 * cpar**2 + 1.0d0
      prm_d = prm_e * rerf
            
      prm_b = - 2 * cpar
      prm_a = - 2.0d0 / 3 * cpar ** 3 + prm_b
      prm_c = cpar * dexp(- (cpar**2)) + prm_d * derf(cpar)
      prm_rc = 1 / prm_c
         
      mfp = mgas * vpar**2 * prm_rc /
     $     (2 * dsqrt(pi) * rmax**2 * pgas)
      return
      end

      
c==============
      double precision function distf(vgas)
c
c     calculating the dist. func. of colliding gas velocity
c
c     == input==
c       vgas ... velocity of gas(m/s)
c
c     This function returns the possibility that
c     the colliding gas velocity < vgas
c     
      real*8 vgas

      real*8 salpha, cpar, derf
      real*8 prm_a, prm_b, prm_c, prm_d, prm_e, prm_rc
      common /ccoldf/ salpha, cpar,
     $     prm_a, prm_b, prm_c, prm_d, prm_e, prm_rc 

      real*8 cgas, cg2, ecg2, fc
      
      cgas = vgas * salpha      
      cg2 =  cgas **2
      ecg2 = dexp(-cg2)
      
      if (cgas .gt. cpar) then
         fc = (prm_a + prm_b * cg2) * ecg2 + prm_c
      else
         fc = prm_d * derf(cgas) -
     $        (2.d0 / 3 * cg2 + prm_e) * cgas * ecg2
      end if
      
      distf = prm_rc * fc

      return
      end
