c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: colgas.F,v 1.7 2001/04/15 15:44:35 nakano Exp $
c
c     routines to obtain the colliding gas speed
c
c     for theoretical backgrounds, see:
c
c     T. Nakano, I. Mori and S. Baba;
c     Appl. Surf. Sci., 113/114 (1997) 642.
c
#include "hisput.h"

c=============
      subroutine colgas(ep, wp, vg)
c
c     Routine to obtain colliding gas speed for the particle 
c     travelling the Maxwellian gas environment.
c
c     Before calling this routine, setcol MUST be executed.
c
c     ==input==
c       ep ... particle energy (in eV)
c       wp ... random number [0..1)
c
c     ==output==
c       vg ... colliding gas speed (in m/s)
c
      real*8 ep, wp, vg

      real*8 epminc, epmaxc, wpmin, wpmax, galpha
      integer idxcol
      common /ccol/ epminc, epmaxc, wpmax, wpmin, galpha, idxcol

      real*8 epl, spf2
      real*8 c, vgsup, vginf, eps
      parameter (eps = 1.0d-1)

      real*8 fvg, v, a
      fvg(v, a) = v**2 * dexp(-a * v**2)

      if (ep .le. epminc) then
         epl = dlog10(epminc)
      else if (ep .le. epmaxc) then
         epl = dlog10(epmaxc)
      else
         epl = dlog10(ep)
      end if

      if (wp .le. wpmin) then
         vg = (wp / wpmin)**(1.0d0/3) * spf2(idxcol, epl, wpmin)

      else if (wp .le. wpmax) then
         vg = spf2(idxcol, epl, wp)

      else if (galpha .eq. 0.0d0) then
         vg = spf2(idxcol, epl, wpmax)

      else
         vg = spf2(idxcol, epl, wpmax)
         c = (1.0d0 - wpmax) / fvg(vg, galpha)
         vginf = vg

 10      continue
            vg = vg * 1.5d0
         if (1.0d0 - c * fvg(vg, galpha) .lt. wp) go to 10
         vgsup = vg

 20      continue
            vg = 0.5d0 * (vgsup + vginf)
            if (1.0d0 - c * fvg(vg, galpha) .gt. wp) then
               vgsup = vg
            else
               vginf = vg
            end if
         if (vgsup - vginf .gt. eps) go to 20
      end if

      return
      end

c=============
      subroutine frpath(ep, wp, fp)
c
c     Routine to obtain mean free path of the particle 
c     travelling the Maxwellian gas environment.
c
c     Before calling this routine, setcol MUST be called
c
c     ==input==
c       ep ... particle energy (in eV)
c       wp ... random number [0..1)
c
c     ==output==
c       fp ... free path (in m)
c
      real*8 ep, wp, fp

      real*8 epminf
      integer idxmfp
      common /cmfp/ epminf, idxmfp

      double precision spf1
      real*8 epl, mfp

      if (ep .lt. epminf) then
C
C     mfp is proportional to vp when vp<<vg
C
         epl = log10(epminf)
         mfp = spf1(idxmfp, epminf)
         mfp = mfp * dsqrt(ep / epminf)
      else
         epl = log10(ep)
         mfp = spf1(idxmfp, epl)
      end if
      fp = - mfp * dlog(1.0d0 - wp)

      return
      end

c=============
      subroutine setcol(tgas, ngas, mgas, mpar, rmax,
     $     ne, nw, emax, emin, wmax, wmin)
c
c     initialization subroutine for functions vgas and frpath 
c
c     inputs
c       tgas  ... gas temperature (K)
c       ngas  ... gas density
c       mgas  ... weight of gas molecule (kg)
c       mpar  ... weight of particle     (kg)
c       rmax  ... maximum collision parameter (m)
c
c       ne    .. number of nodes using on e  axis for spline function
c       nw    ..                          w
c       emax  .. maximum of e0 in eV.
c       emin  .. minimum            (see above)
c       wmax  .. maximun in w axis. at w>wmax, vg is treated as const.
c       wmin  .. minimum in w axis
c
c     variables
c       pgas  ... gas pressure (Pa: actual value)
c
c     This routine sets spline function parameters.
c     Before calling this routine, inispf1 & inispf2 MUST be called.
c
      real*8 tgas, ngas
      real*8 mgas, mpar, rmax
      integer ne, nw
      real*8 emax, emin, wmax, wmin

      real*8 pgas, mfp
      real*8 emrat2, vpar

      real*8 unitch, unitbz, pi
      parameter (unitch = UNITCH)
      parameter (unitbz = UNITBZ)
      parameter (pi     = PIDBL)

      integer nemax, nwmax
      parameter (nemax = NMAXCOL, nwmax = NMAXCOL)
      real*8 lemin, lemax, lestep
      real*8 wpstep

      real*8 vgmax, vgmin, wvgmax
      real*8 ep(nemax), wp(nwmax), vg(nemax, nwmax), fp(nemax)

      integer kep, kwp, kfp
      parameter(kep = KSP2SCA, kwp = KSP2SCA, kfp = KSP1MFP)

      integer idcol, idmfp
      parameter (idcol = ISP2COL, idmfp = ISP1MFP)
      integer idxcol, idxmfp
      real*8 epminf, epminc, epmaxc, wpmin, wpmax, galpha
      common /ccol/ epminc, epmaxc, wpmax, wpmin, galpha, idxcol
      common /cmfp/ epminf, idxmfp

      real*8 distf
      real*8 bfun
      external bfun
      real*8 eps
      parameter(eps = 1.0d-5)
      real*8 vgroot

      integer ie, iw
      real*8 epv

      idxcol = idcol
      idxmfp = idmfp
      epminc = emin
      epmaxc = emax
      epminf = emin

c
c     set spline nodes
c     
      lemin = log10(emin)
      lemax = log10(emax)
      lestep = (lemax - lemin) / (ne - 1)
      
      do 10 ie = 1, ne - 1
         ep(ie) = lemin + lestep * (ie - 1)
 10   continue
      ep(ne) = lemax

      wpmax = wmax
      wpmin = wmin
      wpstep = (wpmax - wpmin) / (nw - 1)

      do 20 iw = 1, nw - 1
         wp(iw) = wpmin + wpstep * (iw - 1)
 20   continue
      wp(nw) = wpmax

c
c     Calculate values at nodes (conditional by Tg=0 or not)
c
      if (tgas .le. 0.0d0) then
         galpha = 0.0d0
         mfp = 1.0d0 / (pi * rmax**2 * ngas)

         do 40 ie = 1, ne
            fp(ie) = mfp
            do 50 iw = 1, nw
               vg(ie, iw) = 0.0d0
 50         continue
 40      continue

      else
         pgas = ngas * unitbz * tgas
         galpha = mgas / ( 2 * unitbz * tgas )

         emrat2 = 2 * unitch / mpar

         do 60 ie = 1, ne
            if (ie .eq. 1) then
               epv = emin
            else if (ie .lt. ne) then
               epv = 10.0d0**ep(ie)
            else
               epv = emax
            end if

            vpar = sqrt( epv * emrat2 )
            call setdist(pgas, tgas, mgas, rmax, vpar, mfp)

            fp(ie) = mfp
c
c     bisection initial range (note that distf depends on epv)
c
            vgmin = 0.0d0
            vgmax = 1.0d4
 70         continue
               vgmax = 2 * vgmax
               wvgmax = distf(vgmax)
            if (wvgmax .lt. wmax) go to 70

c
c     determine vg
c
            do 80 iw = 1, nw
               call setbfun(wp(iw))
               call bisctf(bfun, vgmax, vgmin, eps, vgroot)
               vg(ie, iw) = vgroot
 80         continue

 60      continue

      end if


#ifdef DEBUG
c
c     print spline nodes/values
c
      write(0,9000)
 9000 format('#### debug output of setcol() BEGIN')

      write(0,9010) ngas, tgas, rmax
 9010 format('# Condition --',
     $       ' Ng(m^-3): ', e10.2, 
     $       ' Tg(K): ',    e10.2,
     $       ' Spg(m): ',   e10.2, /,
     $       '#')

      write(0,9020)
 9020 format('# Vp(m/s) and MFP(m/s)')
      do 900 ie = 1, ne
         epv = 10.0d0**ep(ie)
         vpar = sqrt( epv * emrat2 )
         write(0,9030) vpar, fp(ie)
 900  continue
 9030 format(2e14.7)

      write(0,9040)
 9040 format(/, '# Ep, Vp(m/s), Wp and Vg(m/s)')
      do 910 ie = 1, ne
         epv = 10.0d0**ep(ie)
         vpar = sqrt( epv * emrat2 )

         do 920 iw = 1, nw
            write(0,9050) epv, vpar, wp(iw), vg(ie, iw)
 9050       format(4e14.7)
 920     continue

         write(0,*)
 910  continue

      write(0,9070)
 9070 format('#### debug output of setcol() END')
#endif /* DEBUG */

c
c     initialize spline functions
c
      call setspf1(idxmfp, ne, kfp, ep, fp)
      call setspf2(idxcol, nemax, ne, nw, kep, kwp, ep, wp, vg)

      return
      end
      

c======
      subroutine setbfun(w)
c
c     set parameter for bfun
c
      real*8 w
      real*8 wvgas
      common /cgaswp/ wvgas

      wvgas = w

      return
      end


      double precision function bfun(v)
c
c     function to be called from bisect() to obtain the
c     root distf(v) = w
c
      real*8 v
      real*8 wvgas
      common /cgaswp/ wvgas

      real*8 distf
      
      bfun = distf(v) - wvgas
      return
      end

