c     This file is a part of HISPUT, simulation program of
c     sputtering deposition.
c
c     Copyright (c) 1998 Takeo Nakano
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version
c     2 of the License, or (at your option) any later version.
c
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public
c     License along with this program; if not, write to the Free
c     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
c     MA 02111-1307, USA
c
c     $Id: scatter.F,v 1.3 2000/06/13 11:53:03 nakano Exp $
c
c     routines for calculating scattering angle
c
c     **** IMPORTANT NOTICE ***
c     In the routines defined in this file, sp(r) is assumed as a
c     inter-atomic potential. sp(r) should return the value in eV,
c     for given r in angstrom. Of course, sp(r) should be +\infty
c     as r->0, and 0 as r->+\infty.
c
#include "hisput.h"

      subroutine scat(e0, bc, sangle)
c
c     subroutine to obtain scattering angle using spline function.
c     Before calling this function, setsca MUST be executed.
c
c     ==INPUT==
c       e0   ... initial energy of the collision (in e\V)
c       bc   ... collision parameter (in angstroms)
c
c     ==OUTPUT==
c       sangle ... scattering angle (in radian)
c
      real*8 e0, bc, sangle
      integer idxsct
      real*8 emin
      common /csct/ emin, idxsct

      real*8 e0l, spf2

      if (e0 .lt. emin) e0 = emin
      e0l = dlog10(e0)
      sangle = spf2(idxsct, e0l, bc)

      return
      end


      subroutine setsca(ne0, nbc, e0max, e0min, bcmax)
c
c     Prepare the spline function for the scattering problem 
c     in the center-of-mass system. e0 represents the kinetic energy
c     of pserud-particle mass \mu, and bc represents the collision
c     parameter.
c
c     e0min and e0max is min/max of e0, respectively. e0min is needed
c     because spline function is build using log(e0) as a parameter.
c     Hence, e0min should be greater than zero.
c
c     This routine uses dehint routine for integration.
c     Before calling this routine, hiab MUST be executed.
c
c     ==input==
c       ne0   .. number of nodes using on e0 axis for spline function 
c       nbc   ..                          bc
c       e0max .. maximum of e0 in eV.
c       e0min .. minimum            (see above)
c       bcmax .. maxiumum of bc in angstroms (c.f. minimum is 0)
c
      integer ne0, nbc
      real*8 e0max, e0min, bcmax

      real*8 e0lmin, e0lmax, e0lstep, e0lv
      real*8 bcstep, bcv
      integer ie0l, ibc

      integer idsct, idxsct
      parameter (idsct = ISP2SCA )
      real*8 emin
      common /csct/ emin, idxsct

      integer ne0max, nbcmax
      parameter (ne0max = NMAXSCA, nbcmax = NMAXSCA)
      real*8 e0l(ne0max), bc(nbcmax), da(ne0max, nbcmax)
      integer ke0, kbc
      parameter (ke0 = KSP2SCA, kbc = KSP2SCA)

      real*8 sp, scint1, scint2
      external sp, scint1, scint2

      real*8 cbc, ce0
      common /cscint/ cbc, ce0

      real*8 r0u, r0l, r0

      real*8 pi, epsr, epsi, v
      parameter (pi   = PIDBL )
      parameter (epsr = 1.0d-14)
      parameter (epsi = 1.0d-7)

      idxsct = idsct
      emin   = e0min

      e0lmin = log10(e0min)
      e0lmax = log10(e0max)
      e0lstep = (e0lmax - e0lmin) / (ne0 - 1)
      bcstep = bcmax / (nbc - 1)

      r0l = bcmax
 10   continue
      if (sp(r0l) .le. e0max) then
         r0l = r0l / 2
         go to 10
      end if

      do 20 ie0l=1, ne0
         e0lv = e0lmin + (ie0l - 1) * e0lstep
         e0l(ie0l) = e0lv
         ce0 = 10.0d0**e0lv

         do 30 ibc = 1, nbc
            bcv = bcstep * (ibc - 1)
            bc(ibc) = bcv

            if (bcv .eq. 0.0d0) then
               v = 2 * pi
            else
               cbc = bcv

               r0u = bcv
 40            continue
               if (scint2(r0u) .le. 0.0d0) then
                  r0u = 2 * r0u
                  go to 40
               end if

               call bisctf(scint2, r0u, r0l, epsr, r0)
               call dehint(scint1, r0, epsi, v)

            end if
#ifdef DEBUG
            write(0,*) ce0, cbc, r0, cbc * v
#endif
            da(ie0l, ibc) = pi - 2 * bcv * v
 30      continue
 20   continue

      call setspf2(idxsct, ne0max, ne0, nbc, ke0, kbc, e0l, bc, da)

#ifdef DEBUG
      write(0,*) 'spf2 for sangle done'
#endif

      return
      end

      real*8 function scint1(r)
c
c     integrand function for calculation of scattering angle
c     with subroutine scattr
c
      real*8 r, scint2

      scint1 = 1.0d0 / r**2 / dsqrt( scint2(r) )
      return
      end

      real*8 function scint2(r)
c
c     denominator for scint1
c
c     sp is scattering potential in eV, 
c     for distance r in angstrom
c
      real*8 r
      real*8 sp

      real*8 cbc, ce0
      common /cscint/ cbc, ce0
      
      scint2 = 1.0d0 - (cbc/r)**2 - sp(r) / ce0

      return
      end

